/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef TOOL_FILTER_HH_
# define TOOL_FILTER_HH_

# include <wipal/tool/iterator.hh>
# include <wipal/tool/iterable.hh>

namespace tool
{

  /*---------------------.
  | Forward declarations |
  `---------------------*/

  namespace internals
  {

    template <class I, class P, class B1, class B2>
    struct filter_iterator;

  } // End of namespace tool::internals.

  template <class I, class P, class B>
  struct filter;

  /*--------------.
  | Virtual types |
  `--------------*/

  template <class I, class P, class B1, class B2>
  struct types< internals::filter_iterator<I, P, B1, B2> >
  {
    typedef typename I::value_type			value_type;
    typedef filter<I, P, B1>				iterable_type;
  };

  template <class I, class P, class B>
  struct types< filter<I, P, B> >
  {
    typedef internals::filter_iterator<I, P, B, bottom>	iterator;
  };

  /*-------------.
  | Declarations |
  `-------------*/

  namespace internals
  {

    template <class I, class P, class B, class Bottom = bottom>
    struct filter_iterator:
      tt_inherit(public tool::iterator, filter_iterator<I, P, B, Bottom>)
    {
      /// @name Static hierarchies related types
      //@{
      typedef tt_get_exact(Bottom, filter_iterator<I, P, B, Bottom>)
							exact_type;
      typedef tool::iterator<exact_type>		super_type;
      typedef tt_type(value_type, exact_type)		value_type;
      typedef tt_type(iterable_type, exact_type)	iterable_type;
      //@}

      /// tool::iterable compatible constructor.
      filter_iterator(const iterable_type& i, bool end);

      /// @name Abstract method implementations
      //@{
      bool			equal(const filter_iterator& rhs) const;
      void			increment();

      const value_type&		get() const;
      value_type&		get();

      const value_type*		get_ptr() const;
      value_type*		get_ptr();
      //@}

    private:
      void			skip_non_matching_items();

      I				current_;
      const iterable_type*	iterable_;
    };

  } // End of namespace internals.

  template <class InputIterator,
	    class Predicate,
	    class Bottom = tool::bottom>
  struct filter:
    tt_inherit(public tool::iterable, filter<InputIterator,
					     Predicate,
					     Bottom>),
    public Predicate
  {
    filter(const InputIterator& first, const InputIterator& last,
	   const Predicate& p = Predicate ());

  private:
    InputIterator	first_;
    InputIterator	last_;

    friend
      class internals::filter_iterator<InputIterator,
				       Predicate,
				       Bottom,
				       tool::bottom>;
  };

} // End of namespace tool.

# include "filter.hxx"

#endif // ! TOOL_FILTER_HH_
