/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_ADDR_MAPPING_HXX_
# define WIFI_ADDR_MAPPING_HXX_

# include <utility>

# include "addr_mapping.hh"

namespace wifi
{
  inline
  addr_mapping::addr_mapping(const std::string& filename): datafile (filename)
  {
    std::ifstream	f (filename.c_str());

    if (f)
      load(f);
  }

  inline
  unsigned
  addr_mapping::operator [] (const wifi::addr& a)
  {
    typedef std::pair<map_type::iterator, bool> insert_return;

    const size_t	i = map_.size() + 1;
    const insert_return	r = map_.insert(std::make_pair(a, i));

    if (r.second)
      *this << i << '\t' << a << '\n';

    return r.first->second;
  }

  inline
  void
  addr_mapping::load(std::istream& in)
  {
    while (true)
      {
	std::string	a;
	unsigned	i;

	in >> i >> a;
	if (not in or in.eof())
	  break;
	map_.insert(std::make_pair(a, i));
      }
  }

} // End of namespace wifi.

#endif // ! WIFI_ADDR_MAPPING_HXX_
