/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_ADDR_TIMESTAMP_ID_HH_
# define WIFI_FRAME_ADDR_TIMESTAMP_ID_HH_

# include <boost/optional.hpp>

# include <wipal/wifi/frame/unique_id/unique_id.hh>
# include <wipal/wifi/timestamp.hh>
# include <wipal/pcap/frame_descriptor.hh>
# include <wipal/wifi/addr_mapping.hh>
# include <wipal/wifi/frame/dissector/dissector.hh>
# include <wipal/wifi/frame/unique_id/helper_hooks.hh>
# include <wipal/wifi/frame/mgt.hh>

namespace wifi
{

  namespace frame
  {

    namespace internals
    {

      struct dst_tmp_tag {};
      struct src_tmp_tag {};
      struct bss_tmp_tag {};

    } // End of namespace wifi::frame::internals.

# define tt_define_addr_tmp_id(AddrName, AddrIdx)			\
									\
    typedef								\
      unique_id<internals::adapter<unsigned,				\
				   internals::AddrName ## _tmp_tag>,	\
		timestamp>						\
      AddrName ## _tmp_id;						\
									\
    template <>								\
    boost::optional<AddrName ## _tmp_id>				\
    make_unique_id<AddrName ## _tmp_id, prism::header>			\
    (const pcapxx::frame_descriptor&	desc,				\
     addr_mapping&			mapping)			\
    {									\
      tt_make_unique_id_prism_dissect(desc, d,				\
				      internals::id_helper_hooks, h);	\
									\
      if (const timestamp* t = h.timestamp_ptr())			\
	{								\
	  const addr* a = h.addr_ptr(AddrIdx);				\
									\
	  assert(a);							\
	  return AddrName ## _tmp_id (desc.pcap_header()->id,		\
				      mapping[*a], *t);			\
	}								\
      return boost::none_t ();						\
    }

    tt_define_addr_tmp_id(dst, mgt::header::da)
    tt_define_addr_tmp_id(src, mgt::header::sa)
    tt_define_addr_tmp_id(bss, mgt::header::bssid)

# undef tt_define_addr_tmp_id

  } // End of namespace wifi::frame.

} // End of namespace wifi.

#endif // ! WIFI_FRAME_SOURCE_TIMESTAMP_ID_HH_
