/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_MACTIME_TRACKER_HXX_
# define WIFI_MACTIME_TRACKER_HXX_

# include "mactime_tracker.hh"

namespace wifi
{

  template <class Bottom>
  mactime_tracker<Bottom>::mactime_tracker():
    mactime_ (0),
    wrap_count_ (0)
  {
  }

  template <class Bottom>
  mactime_tracker<Bottom>::mactime_tracker(const mactime_type& mactime_start):
    mactime_ (mactime_start),
    wrap_count_ (0)
  {
  }

  template <class Bottom>
  void
  mactime_tracker<Bottom>::tick(const mactime_type& m)
  {
    if (m < mactime_)
      ++wrap_count_;
    mactime_ = m;
  }

  template <class Bottom>
  void
  mactime_tracker<Bottom>::tick(const prism::header*	header,
				size_t			caplen,
				bool			swapped)
  {
    if (caplen < sizeof (prism::header))
      return;

    this->exact().tick(header->mactime.get(swapped));
  }

  template <class Bottom>
  tool::microseconds
  mactime_tracker<Bottom>::microseconds() const
  {
    return tool::microseconds(wrap_count_, mactime_);
  }

  template <class Bottom>
  const typename mactime_tracker<Bottom>::mactime_type&
  mactime_tracker<Bottom>::mactime() const
  {
    return mactime_;
  }

} // End of namespace wifi.

#endif // ! WIFI_MACTIME_TRACKER_HXX_
