/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#include <iostream>
#include <stdexcept>
#include <cstdlib>

#include "config.hh"

#include <wipal/tool/tt_options.hh>
#include <wipal/wifi/frame/filter/intersector.hh>
#include <wipal/wifi/frame/filter/fast_intersector.hh>
#include <wipal/pcap/list_of_traces.hh>
#include <wipal/wifi/frame/unique_id/seqctl_bssid_timestamp.hh>
#include <wipal/wifi/addr_mapping.hh>

namespace
{

  struct intersection_dumper
  {

    intersection_dumper(const tool::opt::options& options): options_ (options)
    {
    }

    template <class Intersector>
    void	operator () (const Intersector& i)
    {
      typedef typename Intersector::const_iterator	iterator;

      const iterator		end = i.end();

      for (iterator j = i.begin(); j != end; ++j)
	{
	  if (options_.print_microseconds)
	    std::cout << j->first.microseconds() << '\t';
	  std::cout << j->first << '\t';
	  if (options_.print_microseconds)
	    std::cout << j->second.microseconds() << '\t';
	  std::cout << j->second << '\n';
	}
    }

  private:
    const tool::opt::options&	options_;
  };

} // End of anonymous namespace.

int
main(int argc, char *const argv[])
{
  tool::opt::options	options = tool::opt::parse_args(argc, argv,
							tt_opt_mapping
							tt_opt_endianness
							tt_opt_prism_noise
							tt_opt_usec,
							2, 0);

  try
    {
      using wifi::frame::filter::provide_intersector;

      typedef pcapxx::list_of_traces<>		list_of_traces;
      typedef wifi::frame::seq_bss_tmp_id	unique_id;

      wifi::addr_mapping	mapping (options.mapping);
      list_of_traces		traces1 (options.input[0]);
      list_of_traces		traces2 (options.input[1]);

      traces1.expect(pcapxx::descriptor<>::PRISM_HEADER);
      traces2.expect(pcapxx::descriptor<>::PRISM_HEADER);

      intersection_dumper	dump (options);

      provide_intersector<unique_id, prism::header,
			  wifi::frame::filter::fast_intersector>
	(traces1.begin(), traces1.end(),
	 traces2.begin(), traces2.end(),
	 mapping, dump, options.prism_noise_means_error, options.endianness);
    }
  catch (const std::exception& e)
    {
      std::cerr << "ERROR: " << e.what() << '.' << std::endl;
      return EXIT_FAILURE;
    }
}
